use crate::ui::UIWidget;
use maud::{Markup, Render, html};

#[allow(non_snake_case)]
pub fn Sized<T: UIWidget + 'static>(height: u32, width: u32, inner: T) -> SizedWidget {
    SizedWidget(Box::new(inner), height, width)
}

pub struct SizedWidget(Box<dyn UIWidget>, u32, u32);

impl Render for SizedWidget {
    fn render(&self) -> Markup {
        self.render_with_class("")
    }
}

impl UIWidget for SizedWidget {
    fn can_inherit(&self) -> bool {
        true
    }

    fn base_class(&self) -> Vec<String> {
        vec![format!("h-{}", self.1), format!("w-{}", self.2)]
    }

    fn extended_class(&self) -> Vec<String> {
        let mut c = self.base_class();
        c.extend_from_slice(&self.0.extended_class());
        c
    }

    fn render_with_class(&self, class: &str) -> Markup {
        if self.0.as_ref().can_inherit() {
            self.0
                .as_ref()
                .render_with_class(&format!("{} {class}", self.base_class().join(" ")))
        } else {
            html! {
                div class=(format!("{} {class}", self.base_class().join(" "))) {
                    (self.0.as_ref())
                }
            }
        }
    }
}
